/*
 * Operation for array of bitree type
 * Teodor Sigaev <teodor@stack.net>
 */


#include "postgres.h"

#include "access/gist.h"
#include "access/itup.h"
#include "access/rtree.h"
#include "utils/elog.h"
#include "utils/palloc.h"
#include "utils/array.h"
#include "utils/builtins.h"
#include "storage/bufpage.h"
#include "access/nbtree.h"

#include "bitree.h"

PG_FUNCTION_INFO_V1(_bitq_eq);
Datum   _bitq_eq(PG_FUNCTION_ARGS);
PG_FUNCTION_INFO_V1(_bitq_req);
Datum   _bitq_req(PG_FUNCTION_ARGS);
PG_FUNCTION_INFO_V1(_bitq_contains);
Datum   _bitq_contains(PG_FUNCTION_ARGS);
PG_FUNCTION_INFO_V1(_bitq_contain);
Datum   _bitq_contain(PG_FUNCTION_ARGS);

static bool
inner_arrbitq_eq( ArrayType *node, bitreenode *q ) {
	bitreenode *cur = ARRPTR(node);
	int4 len = ARRNELEMS(node);
	register int4 i=0;

	if ( len<=0 )
		return false;

	if ( cmp_binode(cur,q) == 0 )
		return true;

	for(i=1;i<len;i++) {
		cur = NEXTVAL(cur);
		if ( cmp_binode(cur,q) == 0 )
			return true;
	}

	return false;		
}

Datum   
_bitq_req(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(1) );
	bool res;

	res = inner_arrbitq_eq(
		arrnode,
		(bitreenode*) PG_GETARG_POINTER(0)
	);
	PG_FREE_IF_COPY(arrnode, 1);
	PG_RETURN_BOOL(res);
}

Datum   
_bitq_eq(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(0) );
	bool res;
	
	res = inner_arrbitq_eq(
		arrnode,
		(bitreenode*) PG_GETARG_POINTER(1)
	);
	PG_FREE_IF_COPY(arrnode, 0);
	PG_RETURN_BOOL(res);
}

static bool
inner_arrbitq_contain( ArrayType *node, bitreenode *mask, bool ignorezero ) {
	bitreenode *cur = ARRPTR(node);
	int4 len = ARRNELEMS(node);
	register int4 i=0;

	if ( len<=0 )
		return false;

	if ( inner_bitq_contain(cur,mask,ignorezero) )
		return true;

	for(i=1;i<len;i++) {
		cur = NEXTVAL(cur);
		if ( inner_bitq_contain(cur,mask,ignorezero) )
			return true;
	}

	return false;		
}

Datum   
_bitq_contains(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(1) );
	bool res;
	
	res = inner_arrbitq_contain(
		arrnode,
		(bitreenode*) PG_GETARG_POINTER(0),
		false
	);
	PG_FREE_IF_COPY(arrnode, 1);
	PG_RETURN_BOOL(res);
}

Datum   
_bitq_contain(PG_FUNCTION_ARGS) {
	ArrayType *arrnode = DatumGetArrayTypeP( PG_GETARG_DATUM(0) );
	bool res;
	
	res = inner_arrbitq_contain(
		arrnode,
		(bitreenode*) PG_GETARG_POINTER(1),
		false
	);
	PG_FREE_IF_COPY(arrnode, 0);
	PG_RETURN_BOOL(res);
}


