#ifndef __PG__ENTREE_H__
#define __PG__ENTREE_H__


typedef uint16  leveltype;
typedef uint64	signlevtype;

#define MAXCHILD     (  ( 1 << sizeof(leveltype) * 8) - 1 )
#define MAXSIGNCHILD ( sizeof(signlevtype) * 8 )

typedef struct {
	int4		len;
	leveltype 	id[1];
} entreenode;

#define CALCSIZE(nodelen) ( sizeof(leveltype)*(nodelen) + sizeof(int4) )
#define GETNODELEN(tn)    ( ( ((entreenode*)tn)->len - sizeof(int4) ) / sizeof(leveltype) )
#define GETENTRY(vec,pos) ((char *) DatumGetPointer(((GISTENTRY *) VARDATA(vec))[(pos)].key))

typedef struct {
	uint16		size;
	leveltype    	var[1];
} levelquery;
#define LQ_HDRSIZE	( sizeof(uint16) )
#define LQ_CALCSIZE(l) ( sizeof(leveltype)*(l) + LQ_HDRSIZE )

typedef struct {
	int4 len;
	int4 size;
	levelquery	query[1];
} treequery;
#define TQ_HDRSIZE	( sizeof(int4)*2 )
#define LQ_NEXT(x)	( (levelquery*)( ((char*)(x)) + LQ_CALCSIZE( ((levelquery*)(x))->size) ) )

typedef struct {
	int4            len;
	signlevtype	sign[1];
} signnode;
#define SN_HRDSIZE	( sizeof(int4) )
#define SN_CALCSIZE(nodelen) ( sizeof(signlevtype)*(nodelen) + SN_HRDSIZE )
#define SN_GETLEN(x)  ( ( ((signnode*)(x))->len - SN_HRDSIZE )/sizeof(signlevtype) )
#define ARRNELEMS(x)  ArrayGetNItems( ARR_NDIM(x), ARR_DIMS(x))
#define ARRPTR(x)  ( (entreenode*) ARR_DATA_PTR(x) )
#define NEXTVAL( tnptr ) ( (entreenode*)( (char*)tnptr + INTALIGN( VARSIZE(tnptr) ) ) )

#define NUMLEN	12
#define PRS_WAITNUM	1
#define PRS_INNUM	2
#define PRS_WAITDELIM	3
#define PRS_WAITLEVEL	4

#ifndef max
#define max(a,b)                ((a) >  (b) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)                ((a) <= (b) ? (a) : (b))
#endif
#ifndef abs
#define abs(a)                  ((a) <  (0) ? -(a) : (a))
#endif

/*   
 * return 1 if n1 > n2
 * return -1  if n1 < n2
 */
int cmp_ennode(entreenode *n1, entreenode *n2);
bool inner_en_isparent(entreenode *parent, entreenode *child); 
bool inner_entq_contain(entreenode *node, treequery *mask );
Datum   entree2query(PG_FUNCTION_ARGS);

/* debug */
char* printennode( entreenode *n );
#endif
